#   -*- coding: utf-8 -*-
#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010-2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import sys

from fabric.api import local


HERE = os.path.dirname(os.path.dirname(__file__))
sys.path = [
    HERE, os.path.join(HERE, 'src'), os.path.join(HERE, 'lib')] + sys.path


def manage(command, *args):
    """Run manage.py command"""
    local("virtualenv/bin/python django_project/manage.py {0} {1}".format(
        command, " ".join(args)), capture=False,)


def test(*args, **kwargs):
    """Run unit tests."""
    # XXX: `fabric.contrib.django.settings_module` is of no help here
    # because this module is also called `django` so we can't use
    # `from django.conf import settings`.
    from django_project.settings import INSTALLED_APPS
    possible_labels = set(INSTALLED_APPS + ['src'])

    cmd_args = []
    if kwargs.pop('--noinput', False):
        cmd_args.append('--noinput')
    cmd_args.extend(list(args))
    cmd_args.extend('%s=%s' % i for i in kwargs.iteritems())

    has_test_labels = False
    for app in possible_labels:
        if app in args:
            has_test_labels = True
            break

    if not has_test_labels:
        # If there are no labels defined, run all tests.
        cmd_args.append('src')

    manage('test', *cmd_args)


def runserver(port=8000):
    manage("runserver", "0.0.0.0:"+str(port))


def syncdb():
    manage("syncdb", "--migrate")


def run(port=8000):
    syncdb()
    runserver(port)


def coverage(branch=False, xml=False):
    cov = lambda args: local("virtualenv/bin/python -m coverage " + args,
                             capture=False)

    local("rm -rf .coverage coverage/ coverage.xml")

    cov("run {0} django_project/manage.py test src".format(
        '--branch' if branch else ''))
    cov("combine")
    if xml:
        cov("xml --include='src/*' -o coverage.xml")
    else:
        cov("html -d coverage/ --include='src/*'")
