import unittest
from django.conf import settings
from httplib2 import ServerNotFoundError
from mock import patch
from clickreviews.utilities import verify_click_package_published


class VerifyPackageTestCase(unittest.TestCase):

    def patch_httplib2_request(self):
        patcher = patch('clickreviews.utilities.Http.request')
        self.addCleanup(patcher.stop)
        return patcher.start()

    def patch_click_api_uri(self, uri="http://example.com/api/"):
        patcher = patch.multiple(settings, CLICK_INDEX_API_URI=uri)
        self.addCleanup(patcher.stop)
        return patcher.start()

    def test_click_verify(self):
        mock_request = self.patch_httplib2_request()
        mock_request.return_value = ({
            'content-length': '1152',
            'status': '200',
        }, '')
        self.patch_click_api_uri(uri="http://example.com/api/")

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertTrue(result)
        mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )

    def test_click_verify_false(self):
        mock_request = self.patch_httplib2_request()
        mock_request.return_value = ({
            'content-length': '1152',
            'status': '404',
        }, '')
        self.patch_click_api_uri(uri="http://example.com/api/")

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertFalse(result)
        mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )

    def test_click_verify_error(self):
        mock_request = self.patch_httplib2_request()
        mock_request.side_effect = ServerNotFoundError
        self.patch_click_api_uri(uri="http://example.com/api/")

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertFalse(result)
        mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )
