"""Helpers for various network-related functionality."""
import logging
from django.conf import settings
from httplib2 import (
    Http,
    HttpLib2Error,
)


def verify_click_package_published(package_name):
    """Verify that a given package is published in the click index.

    :return: Whether the package was found or not.
    :rtype: bool
    """
    try:
        result = Http().request(
            settings.CLICK_INDEX_API_URI.rstrip('/') +
            "/package/{}".format(package_name), 'HEAD')
        return result[0]['status'] == '200'
    except HttpLib2Error:
        logger = logging.getLogger('clickreviews.utilities')
        logger.exception("Error querying click-package-index.")
        return False
