from django.conf import settings
from django.db import models

from south.modelsinspector import add_introspection_rules


# custom db type field to accomodate postgresql's debversion field
# if not postgresql, just acts like a normal CharField
class DebVersionField(models.CharField):
    def __init__(self, *args, **kwargs):
        super(DebVersionField, self).__init__(*args, **kwargs)

    def db_type(self, connection=None):
        engine = settings.DATABASES['default']['ENGINE']

        if 'postgresql_psycopg2' in engine:
            return 'debversion'
        else:
            kwargs = {}
            if connection:
                kwargs['connection'] = connection
            return super(DebVersionField, self).db_type(**kwargs)


class IntegerRangeField(models.IntegerField):
    def __init__(self, verbose_name=None, name=None, min_value=None,
                 max_value=None, **kwargs):
        self.min_value, self.max_value = min_value, max_value
        return models.IntegerField.__init__(self, verbose_name, name, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'min_value': self.min_value, 'max_value': self.max_value}
        defaults.update(kwargs)
        return super(IntegerRangeField, self).formfield(**defaults)


add_introspection_rules([], ["^core\.fields\.DebVersionField"])
add_introspection_rules([], ["^core\.fields\.IntegerRangeField"])
