#   -*- coding: utf-8 -*-
#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010-2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Django-preflight checks."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = []

import logging

import launchpadlib
import openid
import piston.utils
import preflight
import south
from django.conf import settings

from core.utilities import web_services


logger = logging.getLogger(__name__)


class RNRPreflight(preflight.Preflight):

    def versions(self):
        return [
            {'name': 'bzr-revno', 'version': self.get_bzr_revno()},
            {'name': 'openid', 'version': openid.__version__},
            {'name': 'launchpadlib', 'version': launchpadlib.__version__},
            {'name': 'piston', 'version': piston.utils.__version__},
            {'name': 'preflight', 'version': preflight.__version__},
            {'name': 'South', 'version': south.__version__},
        ]

    def get_bzr_revno(self):
        """Current deployed revno (if available)."""
        revno = None
        try:
            from bzrlib.errors import NotBranchError, NoWorkingTree
            from bzrlib.workingtree import WorkingTree
            try:
                t = WorkingTree.open_containing(__file__)[0]
                revno = t.branch.revno()
            except (NotBranchError, NoWorkingTree):
                pass
            except Exception:
                logger.exception(
                    "bzrlib and branch available but revno retrieval failed.")
        except ImportError:
            pass
        return revno

    def authenticate(self, request):
        return (request.user.is_authenticated() and
                bool(request.user.groups.filter(name='rnr-developers')))

    def check_identity_provider_api_access(self):
        """
        Make sure that the access to the Ubuntu SSO API is correct.

        """
        sso = web_services.ssoclient
        open_id = settings.SSO_V2_API_TOKEN_DATA['consumer_key']
        return sso.account_details(open_id)

    def check_launchpad_api_access(self):
        """
        Make sure that the access to the Launchpad API (anonymous) works.

        """
        # Pull one data item from Launchpad
        bug = web_services.launchpad_service.bugs[1]
        return bug.id == 1

    def check_sca_validation(self):
        pubs = web_services.get_sca_publishings_for('maverick', 'i386')
        return type(pubs) == dict


preflight.register(RNRPreflight)
