#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from time import time

from reviewsapp.middleware.txstatsd_client import statsd_client as client


class GraphiteMiddleware(object):
    """
    Gather general request/response statistics.
    """
    def __init__(self):
        client.connect()

    def process_response(self, request, response):
        if hasattr(request, 'time_started'):
            duration = time() - request.time_started
            client.timing('requests', duration)
            key = 'status_codes.{0}'.format(response.status_code)
            client.timing(key, duration)
        return response

    def process_request(self, request):
        request.time_started = time()
        return None

    def process_exception(self, request, exception):
        if hasattr(request, 'time_started'):
            duration = time() - request.time_started
            client.timing('exceptions', duration)
            key = 'exceptions.{0}'.format(exception.__class__.__name__)
            client.timing(key, duration)
        return None
