#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A response middleware that uses oops reports for soft timeouts."""

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'SoftTimeoutMiddleware',
]

import logging
from time import time

from django.conf import settings
from django.core.exceptions import MiddlewareNotUsed
from django.db import connection


class SoftTimeoutMiddleware(object):
    def __init__(self):
        if settings.SOFT_REQUEST_TIMEOUT == 0:
            raise MiddlewareNotUsed()

    def process_request(self, request):
        request._start_time = time()

    def process_response(self, request, response):
        if hasattr(request, '_start_time'):
            total = time() - request._start_time
            if total > settings.SOFT_REQUEST_TIMEOUT:
                logging.warn('Request took %.3f seconds to return' % total)
                for query in connection.queries:
                    logging.warn("time: %(time)s sql: %(sql)s" % query)
                request.environ['dump-oops'] = 'yes'
        return response
