#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
import socket

from django.conf import settings

from txstatsd.client import UdpStatsDClient


class StatsdClient(UdpStatsDClient):

    def _get_fully_qualified_stat(self, key):
        hostname = socket.gethostname().replace('.', '_')
        instance_type = settings.TXSTATSD_INSTANCE_TYPE
        return 'ca.{0}.rnr.{1}.{2}'.format(instance_type, hostname, key)

    def increment(self, key, count=1, rate=1):
        key = self._get_fully_qualified_stat(key)
        stat = '{0}:{1}|m|@{2}'.format(key, count, rate)
        super(StatsdClient, self).write(stat)

    def timing(self, key, time, rate=1):
        key = self._get_fully_qualified_stat(key)
        stat = '{0}:{1}|ms|@{2}'.format(key, time, rate)
        super(StatsdClient, self).write(stat)


statsd_client = StatsdClient(settings.TXSTATSD_HOST, settings.TXSTATSD_PORT)
