from __future__ import absolute_import
from django.test import TestCase

from reviewsapp.api.urls import urlpatterns, djson13_url


class URLEmitterFormatTestCase(TestCase):

    def test_all_urls_use_djson13_emitter(self):
        for url in urlpatterns:
            regex = url.regex.pattern
            em_format = url.default_args.get('emitter_format')
            error_msg = "%s `emitter_format` is %s" % (regex, em_format)
            self.assertEqual('djson13', em_format, error_msg)


class DJSON13URLTestCase(TestCase):

    def setUp(self):
        super(DJSON13URLTestCase, self).setUp()
        self.view = lambda request: None

    def test_default_emitter_is_set(self):
        url = djson13_url(r'some', self.view, None, prefix='')
        self.assertEqual(url.default_args, {'emitter_format': 'djson13'})

    def test_default_kwargs_are_updated_not_replaced(self):
        url = djson13_url(r'some', self.view, {'a': 1, 'b': 2}, prefix='')
        expected = {'a': 1, 'b': 2, 'emitter_format': 'djson13'}
        self.assertEqual(url.default_args, expected)

    def test_emitter_is_forced(self):
        url = djson13_url(
            r'some', self.view, {'a': 1, 'emitter_format': 'xml'}, prefix='')
        expected = {'a': 1, 'emitter_format': 'djson13'}
        self.assertEqual(url.default_args, expected)
