#   -*- coding: utf-8 -*-
#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010-2011 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the 'update_stats' command."""

__metaclass__ = type
__all__ = [
    'UpdateStatsCommandTestCase',
]

from django.core.management import call_command
from mock import patch

from reviewsapp.tests.factory import TestCaseWithFactory
from reviewsapp.models import SoftwareItem


class UpdateStatsCommandTestCase(TestCaseWithFactory):
    @patch('reviewsapp.models.Review.update_wilson_rating')
    @patch('reviewsapp.models.SoftwareItemInRepository.update_stats')
    def test_calls_update_stats_on_all_items(self, mock_update_stats,
                                             mock_update_wilson):
        # When our test suite runs on Jenkins with the xmltestrunner,
        # there are SoftwareItems already in the DB (perhaps some tests
        # with transactions not being rolled back)
        SoftwareItem.objects.all().delete()
        nitems = 20
        for i in range(nitems):
            self.factory.makeSoftwareItemInRepository()

        call_command('update_stats', verbosity=0, update_wilson=False)

        self.assertEqual(nitems, mock_update_stats.call_count)
        self.assertEqual(0, mock_update_wilson.call_count)

    @patch('reviewsapp.models.Review.update_wilson_rating')
    def test_calls_updates_all_wilson(self, mock_update_wilson):
        # When our test suite runs on Jenkins with the xmltestrunner,
        # there are SoftwareItems already in the DB (perhaps some tests
        # with transactions not being rolled back)
        SoftwareItem.objects.all().delete()
        nitems = 20
        for i in range(nitems):
            self.factory.makeReview()

        call_command('update_stats', verbosity=0, update_wilson=True)

        self.assertEqual(nitems, mock_update_wilson.call_count)
