#   This file is part of Software Center Ratings and Reviews
#   Copyright (C) 2010 Canonical Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as
#   published by the Free Software Foundation, either version 3 of the
#   License, or (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import absolute_import

__metaclass__ = type
__all__ = [
    'overview',
    'die',
]

from time import sleep

from django.http import HttpResponse
from django.shortcuts import render_to_response

from reviewsapp.models import ReviewModeration
from reviewsapp.decorators import timed_call


@timed_call
def overview(request):
    """Show an overview of the reviews application.

    :param request: The Django request object.
    :return: A Django response.
    """
    # Find the matching reviews requiring moderation.
    review_moderation_count = ReviewModeration.objects.filter(
        status=ReviewModeration.PENDING_STATUS).count()

    context = {
        'review_moderation_count': review_moderation_count,
    }
    return render_to_response('reviewsapp/overview.html', context)


def die(request):
    """Just trigger an oops report"""
    raise ZeroDivisionError()


def delay(request, amount):
    try:
        amount = float(amount)
    except ValueError:
        return HttpResponse('Invalid delay amount "%s"' % amount)
    sleep(amount)
    return HttpResponse('Slept for %s seconds' % amount)
